# -*- coding: utf-8
RESOURCES = {
    'hg18': {
        'url': 'https://www.ncbi.nlm.nih.gov/grc/human/data?asm=NCBI36',
        'chromosomes': 'http://hgdownload.soe.ucsc.edu/goldenPath/hg18/bigZips/hg18.chrom.sizes',
        'cytoband': 'http://hgdownload.soe.ucsc.edu/goldenPath/hg18/database/cytoBand.txt.gz',
        'refgene': 'http://hgdownload.soe.ucsc.edu/goldenPath/hg18/database/refGene.txt.gz',
        'kgXref': 'http://hgdownload.cse.ucsc.edu/goldenPath/hg18/database/kgXref.txt.gz',
    },
    'hg19': {
        'url': 'https://www.ncbi.nlm.nih.gov/grc/human/data?asm=GRCh37.p13',
        'chromosomes': 'http://hgdownload.soe.ucsc.edu/goldenPath/hg19/bigZips/hg19.chrom.sizes',
        'cytoband': 'http://hgdownload.soe.ucsc.edu/goldenPath/hg19/database/cytoBand.txt.gz',
        'refgene': 'http://hgdownload.soe.ucsc.edu/goldenPath/hg19/database/refGene.txt.gz',
        'kgXref': 'http://hgdownload.cse.ucsc.edu/goldenPath/hg19/database/kgXref.txt.gz',
    },
    'hg38': {
        'url': 'https://www.ncbi.nlm.nih.gov/grc/human/data?asm=GRCh38.p10',
        'chromosomes': 'http://hgdownload.soe.ucsc.edu/goldenPath/hg38/bigZips/hg38.chrom.sizes',
        'cytoband': 'http://hgdownload.soe.ucsc.edu/goldenPath/hg38/database/cytoBand.txt.gz',
        'refgene': 'http://hgdownload.soe.ucsc.edu/goldenPath/hg38/database/refGene.txt.gz',
        'kgXref': 'http://hgdownload.cse.ucsc.edu/goldenPath/hg38/database/kgXref.txt.gz',
    }
}

HGNC_GENES = 'http://tinyurl.com/mqot6vl'
