# -*- coding: utf-8 -*-
# Generated by Django 1.11.8 on 2018-03-30 13:37
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('genome', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='gene',
            name='symbol',
            field=models.CharField(db_index=True, max_length=255),
        ),
        migrations.AlterUniqueTogether(
            name='gene',
            unique_together=set([('chromosome', 'symbol')]),
        ),
    ]
